import matplotlib.patches
import numpy as np
from library_dicom.dicom_processor.model.csv_reader.Roi import Roi

"""Derivated Class for manual Polygon ROI of PetCtViewer.org

Returns:
    [RoiPolygon] -- Polygone ROI
"""
class RoiPolygon(Roi):

    def __init__(self, axis, first_slice, last_slice, roi_number, type_number, list_point, volume_dimension):
        super().__init__(axis, first_slice, last_slice, roi_number, type_number, list_point, volume_dimension)
        self.list_points = self.calculateMaskPoint()

    def calculateMaskPoint(self):
        """list of [x,y,z] coordonates of a ROI 

        Returns:
            [type] -- [description]
        """
        #np_array_3D = super().get_empty_np_array()
        #x,y,z = np_array_3D.shape

        roi_pixel_matplot = self.__create_closed_polygon()
        list_points = []
        #list_slices = []
        for number_slice in range(self.first_slice - 1  , self.last_slice ) : 
            #print(number_slice)
            point = super().mask_roi_in_slice(roi_pixel_matplot)
            #np_array_3D[:,:,number_of_slices] = mask
            for i in range(len(point)):
                point[i].append(number_slice) 


            list_points.extend(point)
        
        #if (self.axis == 2) : 
           # return super().coronal_to_axial(np_array_3D)
        #elif (self.axis == 3) :
            #return super().sagittal_to_axial(np_array_3D)

        #return np.transpose(np_array_3D.astype(np.uint8), (1,0,2)), list_points, list_slices
        return list_points

    def __create_closed_polygon(self):
        points_array = self.list_point_np
        return matplotlib.patches.Polygon(points_array  , closed = True)


from radiomics.featureextractor import RadiomicsFeatureExtractor 
import SimpleITK as sitk 
from itertools import combinations
import numpy as np
from library_dicom.dicom_processor.tools.threshold_mask import *


"""Tools to extract metrics from PET/MASK using pyradiomics
"""


def get_center_of_mass(mask_path, thresh = False, pet_path = None):
    """Get the list of every ROIs center

    Args:
        mask_path ([str]): [nifti MASK path ]
        thresh (bool, optional): [True if want to threshold MASK ]. Defaults to False.
        pet_path ([str], optional): [If thresh is True, nifti PET path]. Defaults to None.

    Returns:
        [list]: [Return a list of list of x,y,z physical coordonates of every ROIs center.]
    """
    center = []

    if pet_path != None : 
        pet_img = sitk.ReadImage(pet_path)
        pet_array = sitk.GetArrayFromImage(pet_img).transpose()


    mask_img = sitk.ReadImage(mask_path)
    mask_array = sitk.GetArrayFromImage(mask_img).transpose()


    type_ = None 
    if len(mask_array.shape) != 3 : 
        type_ = '4D'
        origin = mask_img.GetOrigin()[0:3]
        direc = mask_img.GetDirection()
        direction = direc[0:3] + direc[4:7] + direc[8:11]
        spacing = mask_img.GetSpacing()[0:3]
        size = mask_img.GetSize()[0:3]

        pet_arr = np.random.randint(10, size=(size)).transpose()
        #pet_arr = np.ones(size).transpose()
        pet_sitk = sitk.GetImageFromArray(pet_arr)
        pet_sitk.SetOrigin(origin)
        pet_sitk.SetDirection(direction)
        pet_sitk.SetSpacing(spacing)


    else : 
        type_ = '3D'
        origin = mask_img.GetOrigin()
        direction = mask_img.GetDirection()
        spacing = mask_img.GetSpacing()
        size = mask_img.GetSize() 

        #pet_arr = np.ones(size).transpose()
        pet_arr = np.random.randint(10, size=(size)).transpose()
        pet_sitk = sitk.GetImageFromArray(pet_arr)
        pet_sitk.SetOrigin(origin)
        pet_sitk.SetDirection(direction)
        pet_sitk.SetSpacing(spacing)


    if type_ == '3D' : 
        if thresh == False : 
            number_of_roi = int(np.max(mask_array))
            for i in range(1, number_of_roi + 1) : 
                extractor = RadiomicsFeatureExtractor()
                results = extractor.execute(pet_sitk, mask_img, label = i)
                x, y, z = results['diagnostics_Mask-original_CenterOfMass']
                center.append([x,y,z])

        elif thresh == True : 
            print("mask_seuillé")
            if np.max(mask_array) == 1.0 : 
                mask_threshold = threshold_matrix(mask_array, pet_array, 0.41)
                mask_threshold = mask_threshold.transpose()
                new_mask = sitk.GetImageFromArray(mask_threshold)
                new_mask.SetOrigin(origin)
                new_mask.SetDirection(direction)
                new_mask.SetSpacing(spacing)
                extractor = RadiomicsFeatureExtractor()
                results = extractor.execute(pet_img, new_mask)
                x, y, z = results['diagnostics_Mask-original_CenterOfMass']
                center.append([x,y,z])

            else : 
            
                mask_threshold = threshold_matrix(mask_array, pet_array, 0.41)
                mask_threshold = mask_threshold.transpose()
                new_mask = sitk.GetImageFromArray(mask_threshold)
                new_mask.SetOrigin(origin)
                new_mask.SetDirection(direction)
                new_mask.SetSpacing(spacing)
                for i in range(1, number_of_roi + 1) : 
                    extractor = RadiomicsFeatureExtractor()
                    results = extractor.execute(pet_sitk, new_mask, label = i)
                    x, y, z = results['diagnostics_Mask-original_CenterOfMass']
                    center.append([x,y,z])


    if type_ == '4D' : 
        if thresh == False : 
            for i in range(mask_array.shape[3]):
                mask_3d = mask_array[:,:,:,i]
                mask_3d = mask_3d.transpose()
                new_mask = sitk.GetImageFromArray(mask_3d)
                new_mask.SetOrigin(origin)
                new_mask.SetDirection(direction)
                new_mask.SetSpacing(spacing)
                extractor = RadiomicsFeatureExtractor()
                results = extractor.execute(pet_sitk, new_mask)
                x, y, z = results['diagnostics_Mask-original_CenterOfMass']
                center.append([x,y,z])

        elif thresh == True : 
            mask_4d_threshold = threshold_matrix(mask_array, pet_array, 0.41)
            print("mask seuillé")
            for i in range(mask_array.shape[3]):
                mask_3d = mask_4d_threshold[:,:,:,i]
                x,y,z = np.where(mask_3d != 0)
                if len(x) != 0 : 
                    mask_3d = mask_3d.transpose()
                    new_mask = sitk.GetImageFromArray(mask_3d)
                    new_mask.SetOrigin(origin)
                    new_mask.SetDirection(direction)
                    new_mask.SetSpacing(spacing)
                    extractor = RadiomicsFeatureExtractor()
                    results = extractor.execute(pet_img, new_mask)
                    x, y, z = results['diagnostics_Mask-original_CenterOfMass']
                    center.append([x,y,z])
                else : pass

    return center 


def distance(coord_A, coord_B):
    """calcul euclidian distance between 2 points in 3D 

    Args:
        coord_A ([liste]): [[x,y,z] coordonate point A]
        coord_B ([liste]): [[x,y,z] coordonate point A]

    Returns:
        [float]: [Return euclidian distance]
    """
    return np.sqrt((coord_A[0]-coord_B[0])**2 + (coord_A[1]-coord_B[1])**2 + (coord_A[2]-coord_B[2])**2)


def calcul_distance_max(list_center):
    """Calcul the maximum distance between two ROIs

    Args:
        list_center ([list]): [Return a list of list of x,y,z physical coordonates of every ROIs center]

    Returns:
        [float]: [Return maximum distance, in mm]
    """
    if len(list_center) == 1 : #only 1 roi
        return 0
    else : 
        comb = combinations(list_center, 2)
        comb_liste = list(comb)
        liste_distance = []
        for combinaison in comb_liste : 
            point_A = combinaison[0]
            point_B = combinaison[1]
            liste_distance.append(distance(point_A, point_B))

        maxi = np.max(liste_distance)
  
        return np.round(maxi,2)



def get_bigger_roi_number(mask_path, pet_path):
    img_mask = sitk.ReadImage(mask_path)
    img_pet = sitk.ReadImage(pet_path)
    mask_array = sitk.GetArrayFromImage(img_mask).transpose()
    pet_array = sitk.GetArrayFromImage(img_pet).transpose()

    mask_array = threshold_matrix(mask_array, pet_array, 0.41)
    pixel_spacing = img_pet.GetSpacing() #[x,y,z]

    volume_voxel = pixel_spacing[0]*pixel_spacing[1]*pixel_spacing[2] * 10**(-3)

    if len(mask_array.shape) != 3 : 
        number_of_roi = mask_array.shape[3]
    else : 
        number_of_roi = 1

    volume = []

    for i in range(number_of_roi) : 
        if number_of_roi != 1 :
            number_pixel = len(np.where(mask_array[:,:,:,i] == 1)[0])
        else : 
            number_pixel = len(np.where(mask_array == 1)[0])

        volume.append(volume_voxel * number_pixel)

    volume_max = np.max(volume)
    roi_max = volume.index(volume_max)

    if number_of_roi != 1 : 
        roi = mask_array[:,:,:,roi_max]

    else : roi = mask_array

    x,y,z = np.where(roi == 1)

    suv_values = []
    for j in range(len(x)):
        suv_values.append(pet_array[x[j],y[j],z[j]])

    return roi_max, np.round(np.max(suv_values), 2)



def get_diameter(mask_path, pet_path, number_bigger_roi) : 
    """get 2D and 3D diameter of the biggest ROI in MASK

    Args:
        mask_path ([str]): [Nifti MASK path]
        pet_path ([str]): [Nifti PET path]
        number_bigger_roi ([int]): [Number of the bigger ROI (in volume)]

    Returns:
        [tuple]: [Return 2D and 3D float diameter]
    """
    #PET
    img_pet = sitk.ReadImage(pet_path)
    origin = img_pet.GetOrigin()
    direction = img_pet.GetDirection()
    spacing = img_pet.GetSpacing()
    pet_array = sitk.GetArrayFromImage(img_pet).transpose()
    size = pet_array.shape

    #MASK
    img_mask = sitk.ReadImage(mask_path)
    array_mask = sitk.GetArrayFromImage(img_mask).transpose()

    #seuillage 41% 
    array_mask = threshold_matrix(array_mask, pet_array, 0.41)
    
    if len(array_mask.shape) != 3 :
        bigger_roi = array_mask[:,:,:,number_bigger_roi - 1]

    else : 
        if np.max(array_mask) == 1.0 : 
            bigger_roi = array_mask
        else  : 
    
            x,y,z = np.where(array_mask == number_bigger_roi - 1)
            bigger_roi = np.zeros(size)
            bigger_roi[x,y,z] = 1

 
    new_mask_img = sitk.GetImageFromArray(bigger_roi.transpose())
    new_mask_img.SetOrigin(origin)
    new_mask_img.SetDirection(direction)
    new_mask_img.SetSpacing(spacing)

    extractor = RadiomicsFeatureExtractor()
    results = extractor.execute(img_pet, new_mask_img)
    diameter_3d = results['original_shape_Maximum3DDiameter']
    diameter_2d = results['original_shape_Maximum2DDiameterSlice']
    return float(diameter_2d) , float(diameter_3d)


import numpy as np
import matplotlib.pyplot as plt

from library_dicom.dicom_processor.model.csv_reader.CsvReader import CsvReader
from library_dicom.dicom_processor.model.csv_reader.RoiFactory import RoiFactory
from library_dicom.dicom_processor.model.csv_reader.RoiPolygon import RoiPolygon
from library_dicom.dicom_processor.model.csv_reader.RoiElipse import RoiElipse
from library_dicom.dicom_processor.model.csv_reader.RoiNifti import RoiNifti
from library_dicom.dicom_processor.model.SeriesPT import SeriesPT


class MaskBuilder(CsvReader):
    """Class to build mask from a csv file 

    Arguments:
        CsvReader {[class]} -- [description]
    """

    def __init__(self, csv_path, matrix_size):
        super().__init__(csv_path)
        self.matrix_size=matrix_size
        self.number_of_rois = len(self.details_rois) - 2 #moins ligne SUL + ligne SUClo
        self.mask_array = self.build_mask()

        
    #def initialize_mask_matrix(self):
     #   """build empty 4D numpy array 
      #  """
       # self.mask_array = np.zeros( (self.matrix_size[0], self.matrix_size[1], self.matrix_size[2], self.number_of_rois )).astype(np.uint8)


    def build_mask(self):
        """build 3D numpy array mask with ROI coordonates from a CSV, put in a 4D matrix

        """

        slice = self.matrix_size[2]
        liste = []

        #construction du mask
        #self.initialize_mask_matrix()
        for number_roi in range(1 ,  self.number_of_rois + 1):
    
            roi_object = RoiFactory(self.details_rois[number_roi], (self.matrix_size[0], self.matrix_size[1], self.matrix_size[2]) , number_roi).read_roi() #.list_points
            #traitement si nifti = > flipper les coordonées 
            if roi_object.type_number == 0 :  
                for point in roi_object.list_points :
                    new_z = slice - 1 - point[2]
                    point[2] = new_z
        
                list_points = roi_object.list_points
                self.details_rois[number_roi]['list_points'] = list_points
                np_array_3D = roi_object.get_mask(list_points) #3D_array AXIAL 
                liste.append(np_array_3D)
            #np.where(np_array_3D == 1) = x, y, z en AXIAL 


            elif roi_object.type_number == 2 or roi_object.type_number == 12 :  
                #coronal
                axial_list_points = self.coronal_list_points_to_axial(roi_object.list_points)
                for point in axial_list_points : 
                    new_z = slice - 1 - point[2]
                    point[2] = new_z 

                self.details_rois[number_roi]['list_points'] = axial_list_points
                np_array_3D = roi_object.get_mask(axial_list_points)
                liste.append(np_array_3D)

                         
            elif roi_object.type_number == 3 or roi_object.type_number == 13 :  
                #saggital
                axial_list_points = self.saggital_list_points_to_axial(roi_object.list_points)
                for point in axial_list_points : 
                    new_z = slice - 1 - point[2]
                    point[2] = new_z 

                self.details_rois[number_roi]['list_points'] = axial_list_points
                np_array_3D = roi_object.get_mask(axial_list_points)
                liste.append(np_array_3D)
                
                
            else : 
                self.details_rois[number_roi]['list_points'] = roi_object.list_points
                np_array_3D = roi_object.get_mask(roi_object.list_points)
                liste.append(np_array_3D)

            
        mask_array = np.stack((liste), axis = 3)
        return mask_array.astype(np.uint8) #liste


    def coronal_list_points_to_axial(self, list_points):
        """ Change the list_points in coronal to axial 
        coronal           axial 
          x                 x
          y                 z
          z                 y
        """
 
        new_list_points = []
        for point in list_points : 
            new_point = []
            new_point.append(point[0])
            new_point.append(point[2])
            new_point.append(point[1]) 
            new_list_points.append(new_point)

        return new_list_points



    def saggital_list_points_to_axial(self, list_points) : 
        """ Change the list_points in saggital to axial
        saggital           axial 
          x                 z
          y                 x
          z                 y
        """
        new_list_points = []
        for point in list_points : 
            new_point = []
            new_point.append(point[2])
            new_point.append(point[0])
            new_point.append(point[1])
            new_list_points.append(new_point)

        return new_list_points



    def calcul_suv(self, nifti_array):
        """calcul SUV Mean, SUV Max and SD from the mask, for each ROI, and put results in a dict
        """
        max_mean = {}
        for number_roi in range(1 , self.number_of_rois + 1):
     
            list_points = self.details_rois[number_roi]['list_points'] #[[x,y,z], [x,y,z],...]
            list_pixels = []
            list_pixels_seuil = []
            results = {}
            for point in list_points :
                list_pixels.append(nifti_array[point[1], point[0], point[2]])



            if list_pixels == [] : #si pas de ROI dessiné
                results['SUV_max'] = float(0)
                results['SUV_mean'] = float(0)
                results['SD'] = float(0)


            else : 
                seuil = self.details_rois['SUVlo']


                if "%" in seuil : 
                    seuil = float(seuil.strip("%"))/100 * np.max(list_pixels)
                else : 
                    seuil = float(seuil)

            
                for i in range(len(list_pixels)):
                    if list_pixels[i] >= seuil : 
                        list_pixels_seuil.append(list_pixels[i])

                if len(list_pixels_seuil) == 0 :
                    results['SUV_max'] = float(0)
                    results['SUV_mean'] = float(0)
                    results['SD'] = float(0)
                else : 
                    #results['pixel_number'] = len(list_pixels_seuil)
                    results['SUV_max'] = round(np.max(list_pixels_seuil), 2)
                    results['SUV_mean'] = round(np.mean(list_pixels_seuil), 2)
                    results['SD'] = round(np.std(list_pixels_seuil, ddof = 1), 2)
            
            max_mean[number_roi] = results

        return max_mean


    #parti check 
    def is_correct_suv(self, nifti_array):
        """check if calculated SUV Mean, SUV Max and SD is correct 

        """

        calculated_suv_max_mean = self.calcul_suv(nifti_array) #dict 
        for number_roi in range(1, self.number_of_rois +1) :

            if (calculated_suv_max_mean[number_roi]['SUV_max'] < float(self.details_rois[number_roi]['suv_max']) - float(0.1) or 
                calculated_suv_max_mean[number_roi]['SUV_max'] > float(self.details_rois[number_roi]['suv_max']) + float(0.1)  ):
                return False


            if (calculated_suv_max_mean[number_roi]['SUV_mean']  < float(self.details_rois[number_roi]['suv_mean']) - float(0.1) or 
                calculated_suv_max_mean[number_roi]['SUV_mean']  > float(self.details_rois[number_roi]['suv_mean']) + float(0.1) ) : 
                return False

            if (calculated_suv_max_mean[number_roi]['SD'] < float(self.details_rois[number_roi]['sd']) - float(0.1) or 
                calculated_suv_max_mean[number_roi]['SD'] > float(self.details_rois[number_roi]['sd']) + float(0.1) ) :
                return False

        return True 


    def ecart_suv_max(self, nifti_array):
        """calculate the difference between calculate_SUV_MAX and csv_SUV_MAX for each ROI 
        """
        liste = []
        calculated_suv_max_mean = self.calcul_suv(nifti_array) #dict 
        for number_roi in range(1, self.number_of_rois +1) :


            if (calculated_suv_max_mean[number_roi]['SUV_max'] < float(self.details_rois[number_roi]['suv_max']) - float(0.1) or 
                calculated_suv_max_mean[number_roi]['SUV_max'] > float(self.details_rois[number_roi]['suv_max']) + float(0.1)  ):
                type_roi = self.details_rois[number_roi]['type_number']
                liste.append(number_roi)
                if type_roi == 1 or type_roi == 2 or type_roi == 3 : 
                    liste.append("POLYGON")

                if type_roi == 11 or type_roi == 12 or type_roi == 13 : 
                    liste.append("ELLIPSE")

                liste.append(float(abs(calculated_suv_max_mean[number_roi]['SUV_max'] - float(self.details_rois[number_roi]['suv_max']))))
            
        return liste 

    def ecart_suv_mean(self, nifti_array) : 
        """calculate the difference between calculate_SUV_MEAN and csv_SUV_MEAN for each ROI 

        """
        liste = []
        calculated_suv_max_mean = self.calcul_suv(nifti_array) #dict 
        for number_roi in range(1, self.number_of_rois +1) :


            if (calculated_suv_max_mean[number_roi]['SUV_mean'] < float(self.details_rois[number_roi]['suv_mean']) - float(0.1) or 
                calculated_suv_max_mean[number_roi]['SUV_mean'] > float(self.details_rois[number_roi]['suv_mean']) + float(0.1)):
                type_roi = self.details_rois[number_roi]['type_number']
                liste.append(number_roi)
                if type_roi == 1 or type_roi == 2 or type_roi == 3 : 
                    liste.append("POLYGON")

                if type_roi == 11 or type_roi == 12 or type_roi == 13 : 
                    liste.append("ELLIPSE")

                liste.append(float(abs(calculated_suv_max_mean[number_roi]['SUV_mean'] - float(self.details_rois[number_roi]['suv_mean']))))            

        return liste 


    def ecart_SD(self, nifti_array) :
        """calculate the difference between calculate_SD and csv_SD for each ROI 

        """
        liste = []
        calculated_suv_max_mean = self.calcul_suv(nifti_array) #dict 
        for number_roi in range(1, self.number_of_rois +1) :

            if (calculated_suv_max_mean[number_roi]['SD'] < float(self.details_rois[number_roi]['sd']) - float(0.1) or 
                calculated_suv_max_mean[number_roi]['SD'] > float(self.details_rois[number_roi]['sd']) + float(0.1)):
                type_roi = self.details_rois[number_roi]['type_number']
                liste.append(number_roi)
                if type_roi == 1 or type_roi == 2 or type_roi == 3 : 
                    liste.append("POLYGON")

                if type_roi == 11 or type_roi == 12 or type_roi == 13 : 
                    liste.append("ELLIPSE")

                liste.append(float(abs(calculated_suv_max_mean[number_roi]['SD'] - float(self.details_rois[number_roi]['sd']))))
                
        return liste 



    def flip_z(self, mask_4D): 
        """flip z axis in the mask matrix 

        """
        slice = mask_4D.shape[2]
    
        liste = []
        for number_roi in range(self.number_of_rois):
            
            new_list_point = []

            liste.append(np.flip(mask_4D[:,:,:,number_roi], axis = 2))
            
            if self.details_rois[number_roi + 1]['list_points'] != [] :
                for point in self.details_rois[number_roi + 1]['list_points'] : 
                    new_z = slice - 1 - point[2]
                    point[2] = new_z

                    new_list_point.append(point)
        
            self.details_rois[number_roi + 1]['list_points'] = new_list_point

        self.mask_array = np.stack((liste), axis = 3)

        return self.mask_array
          

    def is_calcul_sul_correct(self, series_path):
        """check if the SUL in the CSV file and the calculated SUL is the same 

        """
        series_object = SeriesPT(series_path) 
        sul_calculate = round(series_object.calculateSULFactor(),5) 
        sul_csv = self.details_rois['SUL']
        if sul_calculate != sul_csv : 
            return False
        return True 
        

        


import csv

"""CSV Parser for CSV generated by petctviewer.org

Returns:
    [CsvReader] -- [CSV Parser]
"""
class CsvReader():


    def __init__(self, path):
        self.path=path
        self.__load_data()
        self.details_rois = self.get_csv_roi_details()

    def __load_data(self):
        """Load data in this object, during reading, search
        for manual and automatic (nifti) roi declaration part
        """
        with open(self.path, 'r', encoding="utf8", errors='ignore') as csvfile : 
            reader = csv.reader(csvfile)
            csv_data = []
            index = 0
            for row in reader :
                if(row and 'Number of ROIs = ' in row[0]) :
                    number_of_manual_roi = row[0].replace("Number of ROIs = ", "").strip()
                    self.first_line_manual_roi = index
                    self.number_of_manual_roi = int(number_of_manual_roi)
                if(row and 'Number of Nifti ROIs = ' in row[0]) :
                    number_of_nifti_roi = row[0].replace("Number of Nifti ROIs = ", "").strip()
                    self.first_line_nifti_roi = index
                    self.number_of_nifti_roi = int(number_of_nifti_roi) 
                csv_data.append(row) #liste de liste(chaque ligne = liste)
                index += 1

        self.csv_data = csv_data

    def get_rois_results(self):
        """Return CSV lines containing ROIs results
        """
        return self.csv_data[1 : self.csv_data.index([]) - 2 ]

    def get_sum_row(self) :
        """ return csv line containing sum rois results
        """

        return self.csv_data[self.csv_data.index([]) - 2 ]
    
    def get_date(self):
        return self.csv_data[self.csv_data.index([]) - 1 ][1]

    def get_patient_id(self):
        return self.csv_data[self.csv_data.index([]) - 1 ][13]

    def get_manual_rois(self):
        """return manual rois block
        """

        try :
            first_manual_roi = self.first_line_manual_roi + 1
        except AttributeError:
            return ([])
        
        last_manual_row =  first_manual_roi + self.number_of_manual_roi
        return self.csv_data[ first_manual_roi : last_manual_row ]

    def get_nifti_rois(self):
        """ return automatic (nifti) roi block
        """
        
        try :
            first_nifti_row = self.first_line_nifti_roi + 1
        except AttributeError:
            return ([])
        
        nifti_roi_bloc = self.csv_data[first_nifti_row : ] 
        nifti_roi_bloc = nifti_roi_bloc[0 : nifti_roi_bloc.index([])]
        nifti_roi_list = []
        for row in nifti_roi_bloc :
            if len(row) == 1 :
                row.append(' ')
            if "num points = " in row[1] : 
                nifti_roi_list.append(row)
            else : 
                nifti_roi_list[-1] = nifti_roi_list[-1] + row
                if ' ' in row : 
                    row.remove(' ')

        return nifti_roi_list


    def get_SUL(self):
        """return the SUL value of the CSV

        """
        sul =  self.csv_data[0 :  self.csv_data.index([])]
        return float(sul[-1][3])


    def get_SUVlo(self):
        """ return the SUVlo value of the CSV

        """
        last_row = self.csv_data[-1]
        return last_row[0]

    @classmethod 
    def convert_manual_row_to_object(cls, manual_row, results_row):
        """Return a row manual row in an object with ROI details

        Arguments:
            manual_row {list} -- raw, row manual row

        Returns:
            [object] -- object describing the row

        """

        number_point_field = manual_row[4]
        number_point = int( number_point_field.replace(" num points = ", "").strip() )
        point_list_string = manual_row[ 5 : (5 + number_point) ]
        point_list = CsvReader.list_string_to_point_list_int( point_list_string )

        result_answer = {
                'name' : manual_row[0].strip(),
                'roi_number' : results_row[1],
                'first_slice' : int(manual_row[2].strip()),
                'last_slice' : int(manual_row[3].strip()),
                'type_number' : int(manual_row[1].strip()),
                'point_list' : point_list,
                'suv_mean' : results_row[5],
                'sd' : results_row[6],
                'suv_max' : results_row[11]
        }

        return result_answer


    @classmethod
    def convert_nifti_row_to_object(cls, nifti_row, results_row):
        """Return list of point included in the roi

        Arguments:
            nifti_row {list} -- nifti row of the csv

        Returns:
            [list] -- list of point in the roi
        """
        point_list_string = nifti_row[2:]
        point_list = CsvReader.list_string_to_point_list_int( point_list_string )
        result_answer = {
                'name' : nifti_row[0].strip(),
                'roi_number' : results_row[1],
                'first_slice' : 0,
                'last_slice' : 0,
                'type_number' : 0,
                'point_list' : point_list,
                'suv_mean' : results_row[5],
                'sd' : results_row[6],
                'suv_max' : results_row[11]
        }
        return result_answer

    @classmethod
    def list_string_to_point_list_int(cls, point_list_string):
        """
        Transforms point string into list of coordinate (int)
        """
        point_list_string = list(map(str.strip, point_list_string))
        point_list = []
        for point_string in point_list_string:
            list_string_point = point_string.split()
            list_int_point = list(map(int, list_string_point))
            point_list.append(list_int_point)
        return point_list


    def get_csv_roi_details(self):
        """return a dict with each ROI information to build the mask

        """

        rois_results = self.get_rois_results()
        manual_rois = self.get_manual_rois() #list vide si pas de manual
        number_of_manual_rois = len(manual_rois)
        nifti_rois = self.get_nifti_rois() #list vide si pas de nifti
        number_of_nifti_rois = len(nifti_rois)
        details = {}


        for number_roi_nifti in range(number_of_nifti_rois) :
            details[number_roi_nifti + 1] = self.convert_nifti_row_to_object(nifti_rois[number_roi_nifti], rois_results[number_roi_nifti])
        for number_roi_manual in range(number_of_manual_rois):
            details[number_of_nifti_rois + 1 + number_roi_manual]  = self.convert_manual_row_to_object(manual_rois[number_roi_manual], rois_results[number_of_nifti_rois + number_roi_manual])


        details['SUL'] = self.get_SUL()
        details['SUVlo'] = self.get_SUVlo()
        return details 


from library_dicom.dicom_processor.model.Series import Series
import numpy as np

class SeriesCT(Series):
    """Get Series CT Nifti in 16 Bits

    Arguments:
        Series {String} -- Series Location Path
    """

    def __init__(self,path):
        super().__init__(path)

    def get_numpy_array(self):
        numpy_array = super().get_numpy_array()
        return numpy_array.astype(np.int16)



import matplotlib
import numpy as np
from library_dicom.dicom_processor.model.csv_reader.Roi import Roi
import math

"""Derivated Class for manual Elipse ROI of PetCtViewer.org

Returns:
    [RoiElipse] -- Roi Elipse Object
"""
class RoiElipse(Roi):

    def __init__(self, axis, first_slice, last_slice, roi_number, type_number, list_point, volume_dimension):
        super().__init__(axis, first_slice, last_slice, roi_number, type_number, list_point, volume_dimension)
        self.list_points = self.calculateMaskPoint()


    def calculateMaskPoint(self):
        #np_array_3D = super().get_empty_np_array()
        #x,y,z = np_array_3D.shape
        points_array = self.list_point_np 
        x0 = points_array[0][0]
        y0 = points_array[0][1]
        delta_x = points_array[1][0] - x0
        #print("delta_x =", delta_x)
        delta_y = abs(points_array[2][1] - y0)
        #print("delta_y =", delta_y)
        middle = ((self.first_slice + self.last_slice) / 2) - 1
        rad1 = ((self.last_slice - self.first_slice) / 2) 

        list_points = []
        if(rad1 == 0) :
            return list_points

        for number_slice in range(self.first_slice  , self.last_slice - 1 ) : 
            diff0 = abs(middle - number_slice)
            factor = pow(rad1,2) - pow(diff0,2)
            factor = math.sqrt(factor) / rad1
            #print((factor))

            width = factor * delta_x
            height = factor * delta_y
        
            roi_pixel_matplot = self.__create_elipse(2 * width, 2 * height)
            point = super().mask_roi_in_slice(roi_pixel_matplot)

            for i in range(len(point)):
                point[i].append(number_slice)

            #np_array_3D[:,:,number_of_slices] = mask
            
            list_points.extend(point)
        #list_points.append([x0, y0, self.first_slice - 1])
        #list_points.append([x0, y0, self.last_slice - 1])
        #print(list_points)

        #return np.transpose(np_array_3D.astype(np.uint8), (1,0,2)) , list_points, list_slices

        return list_points

    def __create_elipse(self, width, height): 
        points_array = self.list_point_np  
        #width = 2 * abs(points_array[0][0] - points_array[1][0]) #centre_x - est_x 
        #height = 2 * abs(points_array[0][1] - points_array[2][1]) #centre_y - nord_y
        return matplotlib.patches.Ellipse(points_array[0], width, height, angle = 0)



    